package com.fuyuanshen.app.service.impl;

import com.fuyuanshen.common.core.utils.MapstructUtils;
import com.fuyuanshen.common.mybatis.core.page.TableDataInfo;
import com.fuyuanshen.common.mybatis.core.page.PageQuery;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import com.fuyuanshen.app.domain.bo.AppUserRoleBo;
import com.fuyuanshen.app.domain.vo.AppUserRoleVo;
import com.fuyuanshen.app.domain.AppUserRole;
import com.fuyuanshen.app.mapper.AppUserRoleMapper;
import com.fuyuanshen.app.service.IAppUserRoleService;

import java.util.List;
import java.util.Map;
import java.util.Collection;

/**
 * 用户和角色关联Service业务层处理
 *
 * @author Lion Li
 * @date 2025-06-27
 */
@Slf4j
@RequiredArgsConstructor
@Service
public class AppUserRoleServiceImpl implements IAppUserRoleService {

    private final AppUserRoleMapper baseMapper;

    /**
     * 查询用户和角色关联
     *
     * @param userId 主键
     * @return 用户和角色关联
     */
    @Override
    public AppUserRoleVo queryById(Long userId){
        return baseMapper.selectVoById(userId);
    }

    /**
     * 分页查询用户和角色关联列表
     *
     * @param bo        查询条件
     * @param pageQuery 分页参数
     * @return 用户和角色关联分页列表
     */
    @Override
    public TableDataInfo<AppUserRoleVo> queryPageList(AppUserRoleBo bo, PageQuery pageQuery) {
        LambdaQueryWrapper<AppUserRole> lqw = buildQueryWrapper(bo);
        Page<AppUserRoleVo> result = baseMapper.selectVoPage(pageQuery.build(), lqw);
        return TableDataInfo.build(result);
    }

    /**
     * 查询符合条件的用户和角色关联列表
     *
     * @param bo 查询条件
     * @return 用户和角色关联列表
     */
    @Override
    public List<AppUserRoleVo> queryList(AppUserRoleBo bo) {
        LambdaQueryWrapper<AppUserRole> lqw = buildQueryWrapper(bo);
        return baseMapper.selectVoList(lqw);
    }

    private LambdaQueryWrapper<AppUserRole> buildQueryWrapper(AppUserRoleBo bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<AppUserRole> lqw = Wrappers.lambdaQuery();
        lqw.orderByAsc(AppUserRole::getUserId);
        lqw.orderByAsc(AppUserRole::getRoleId);
        return lqw;
    }

    /**
     * 新增用户和角色关联
     *
     * @param bo 用户和角色关联
     * @return 是否新增成功
     */
    @Override
    public Boolean insertByBo(AppUserRoleBo bo) {
        AppUserRole add = MapstructUtils.convert(bo, AppUserRole.class);
        validEntityBeforeSave(add);
        boolean flag = baseMapper.insert(add) > 0;
        if (flag) {
            bo.setUserId(add.getUserId());
        }
        return flag;
    }

    /**
     * 修改用户和角色关联
     *
     * @param bo 用户和角色关联
     * @return 是否修改成功
     */
    @Override
    public Boolean updateByBo(AppUserRoleBo bo) {
        AppUserRole update = MapstructUtils.convert(bo, AppUserRole.class);
        validEntityBeforeSave(update);
        return baseMapper.updateById(update) > 0;
    }

    /**
     * 保存前的数据校验
     */
    private void validEntityBeforeSave(AppUserRole entity){
        //TODO 做一些数据校验,如唯一约束
    }

    /**
     * 校验并批量删除用户和角色关联信息
     *
     * @param ids     待删除的主键集合
     * @param isValid 是否进行有效性校验
     * @return 是否删除成功
     */
    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if(isValid){
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return baseMapper.deleteByIds(ids) > 0;
    }
}
